package com.abcseo.ws2300;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.imageio.ImageIO;
import javax.media.Buffer;
import javax.media.CaptureDeviceInfo;
import javax.media.CaptureDeviceManager;
import javax.media.Manager;
import javax.media.Player;
import javax.media.control.FrameGrabbingControl;
import javax.media.format.VideoFormat;
import javax.media.util.BufferToImage;

/**
 * Grabs a frame from a Webcam, overlays the current date and time, and saves
 * the frame as a JPEG to docRoot + /webcam.png. docRoot is defined in the OpenJ2300.properties file
 * 
 * @author David
 * @version 1.0, 16/01/2004
 */
public class WebCam extends Thread implements WSThread {
	Player player;
	boolean running;
	Configuration config = Configuration.getInstance();
	String docRoot;
	private Logger logger = Logger.getLogger("com.abcseo");

	public WebCam(String device, String docRoot) throws Exception {
		

		CaptureDeviceInfo deviceInfo = CaptureDeviceManager.getDevice(device);
		player = Manager.createRealizedPlayer(deviceInfo.getLocator());
		this.docRoot = docRoot;
		running = true;
	}

	public void run() {

		player.start();
		while (running) {

			try {
				Thread.sleep(1000 * 60 * 10); // 10 minute timeout
			} catch (InterruptedException e) {
				System.out.println("Main Thread Interupted");
			}

			// Grab a frame from the capture device
			FrameGrabbingControl frameGrabber = (FrameGrabbingControl) player
					.getControl("javax.media.control.FrameGrabbingControl");
			Buffer buf = frameGrabber.grabFrame();

			// Convert frame to an buffered image so it can be processed and
			// saved
			Image img = (new BufferToImage((VideoFormat) buf.getFormat())
					.createImage(buf));
			BufferedImage buffImg = new BufferedImage(img.getWidth(null), img
					.getHeight(null), BufferedImage.TYPE_INT_RGB);
			Graphics2D g = buffImg.createGraphics();
			g.drawImage(img, null, null);

			// Overlay curent time on image
			g.setColor(Color.BLUE);
			g.setFont(new Font("Verdana", Font.BOLD, 16));
			g.drawString((new Date()).toString(), 10, 25);

			// Save image to disk as JPG
			try {
				ImageIO.write(buffImg, "jpg", new File(docRoot + "/webcam.jpg"));
			} catch (IOException e) {
				logger.log(Level.SEVERE, "Error writing image to " + docRoot + "/webcam.jpg exception " + e);
				player.close();
				player.deallocate();
			}

		}// while
		logger.log(Level.INFO, "Shutting down webcam");
	}

	public void shutdown() {
		this.interrupt();
		running = false;

		// Stop using webcam
		player.close();
		player.deallocate();
	}
}